#include "ns3/core-module.h"
#include "ns3/network-module.h"
#include "ns3/internet-module.h"
#include "ns3/point-to-point-module.h"
#include "ns3/applications-module.h"
#include "ns3/ipv4-global-routing-helper.h"
#include "ns3/netanim-module.h"

using namespace ns3;
NS_LOG_COMPONENT_DEFINE ("P2P_Topology_Simualtion");

int main (int argc, char *argv[]){
    Time::SetResolution (Time::NS);
    Config::SetDefault("ns3::Ipv4GlobalRouting::RespondToInterfaceEvents", BooleanValue(true));

    NodeContainer p2pNodes;
    p2pNodes.Create(6);
   
    InternetStackHelper stack;
    stack.Install(p2pNodes);

    PointToPointHelper pointToPoint;
    Ipv4AddressHelper Address;

    NodeContainer n01 = NodeContainer (p2pNodes.Get (0), p2pNodes.Get (1));
    NodeContainer n02 = NodeContainer (p2pNodes.Get (0), p2pNodes.Get (2));
    NodeContainer n23 = NodeContainer (p2pNodes.Get (2), p2pNodes.Get (3));
    NodeContainer n34 = NodeContainer (p2pNodes.Get (3), p2pNodes.Get (4));
    NodeContainer n14 = NodeContainer (p2pNodes.Get (1), p2pNodes.Get (4));
    NodeContainer n15 = NodeContainer (p2pNodes.Get (1), p2pNodes.Get (5));
    NodeContainer n45 = NodeContainer (p2pNodes.Get (4), p2pNodes.Get (5));

    Address.SetBase ("10.1.1.0", "255.255.255.0");
    NetDeviceContainer nd01 = pointToPoint.Install(n01);
    Ipv4InterfaceContainer i01 = Address.Assign(nd01);

    Address.SetBase ("10.1.2.0", "255.255.255.0");
    NetDeviceContainer nd02 = pointToPoint.Install(n02);
    Ipv4InterfaceContainer i02 = Address.Assign(nd02);

    Address.SetBase ("10.1.3.0", "255.255.255.0");
    NetDeviceContainer nd23 = pointToPoint.Install(n23);
    Ipv4InterfaceContainer i23 = Address.Assign(nd23);

    Address.SetBase ("10.1.4.0", "255.255.255.0");
    NetDeviceContainer nd34 = pointToPoint.Install(n34);
    Ipv4InterfaceContainer i34 = Address.Assign(nd34);

    Address.SetBase ("10.1.5.0", "255.255.255.0");
    NetDeviceContainer nd14 = pointToPoint.Install(n14);
    Ipv4InterfaceContainer i14 = Address.Assign(nd14);

    Address.SetBase ("10.1.6.0", "255.255.255.0");
    NetDeviceContainer nd15 = pointToPoint.Install(n15);
    Ipv4InterfaceContainer i15 = Address.Assign(nd15);

    Address.SetBase ("10.1.7.0", "255.255.255.0");
    NetDeviceContainer nd45 = pointToPoint.Install(n45);
    Ipv4InterfaceContainer i45 = Address.Assign(nd45);

    uint16_t port = 9; // Discard port (RFC 863)
    OnOffHelper onoff("ns3::UdpSocketFactory", InetSocketAddress(i45.GetAddress(1), port));
    onoff.SetConstantRate(DataRate("2kbps"));
    onoff.SetAttribute("PacketSize", UintegerValue(50));

    ApplicationContainer apps = onoff.Install(p2pNodes.Get(0));
    apps.Start(Seconds(1.0));
    apps.Stop(Seconds(16.0));
   
    NS_LOG_LOGIC ("Create & print rounting table");
    Ipv4GlobalRoutingHelper::PopulateRoutingTables ();

    Ipv4GlobalRoutingHelper g;
    Ptr<OutputStreamWrapper> routingStream = Create<OutputStreamWrapper>
    ("global-routing.routes", std::ios::out);
    g.PrintRoutingTableAllAt (Seconds (2), routingStream);
    g.PrintRoutingTableAllAt (Seconds (12), routingStream);

    Ptr<Node> n1 = p2pNodes.Get(1);
    Ptr<Ipv4> ipv41 = n1->GetObject<Ipv4>();
    // The first ifIndex is 0 for loopback, then the first p2p is numbered 1,
    // then the next p2p is numbered 2
    uint32_t ipv4ifIndex1 = 1;

   Simulator::Schedule(Seconds(5), &Ipv4::SetDown, ipv41, ipv4ifIndex1);
   
    NS_LOG_LOGIC ("Create Animation interface..");
    AnimationInterface anim ("DynamicRouting.xml");
    anim.SetConstantPosition(p2pNodes.Get(0), 10.0, 50.0);
    anim.SetConstantPosition(p2pNodes.Get(1), 30.0, 30.0);
    anim.SetConstantPosition(p2pNodes.Get(2), 30.0, 70.0);
    anim.SetConstantPosition(p2pNodes.Get(3), 50.0, 70.0);
    anim.SetConstantPosition(p2pNodes.Get(4), 70.0, 70.0);
    anim.SetConstantPosition(p2pNodes.Get(5), 90.0, 70.0);

    Simulator::Stop (Seconds (17));
    Simulator::Run ();
  
    Simulator::Destroy ();
    return 0;
}